"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addTailwindConfigFile = exports.addTailwindConfigPathToProject = exports.updateApplicationStyles = exports.addTailwindRequiredPackages = exports.detectTailwindInstalledVersion = exports.normalizeOptions = void 0;
const semver_1 = require("@nx/devkit/src/utils/semver");
const tailwind_dependencies_1 = require("../versions/tailwind-dependencies");
const semver_2 = require("semver");
const devkit_1 = require("@nx/devkit");
const node_path_1 = require("node:path");
function normalizeOptions(options) {
    return {
        ...options,
        buildTarget: options.buildTarget || 'build',
    };
}
exports.normalizeOptions = normalizeOptions;
function detectTailwindInstalledVersion(tree) {
    const { dependencies, devDependencies } = (0, devkit_1.readJson)(tree, 'package.json');
    const tailwindVersion = dependencies?.tailwindcss ?? devDependencies?.tailwindcss;
    if (!tailwindVersion) {
        return undefined;
    }
    const version = (0, semver_1.checkAndCleanWithSemver)('tailwindcss', tailwindVersion);
    if ((0, semver_2.lt)(version, '2.0.0')) {
        throw new Error(`The Tailwind CSS version "${tailwindVersion}" is not supported. Please upgrade to v2.0.0 or higher.`);
    }
    return (0, semver_2.lt)(version, '3.0.0') ? '2' : '3';
}
exports.detectTailwindInstalledVersion = detectTailwindInstalledVersion;
function addTailwindRequiredPackages(tree) {
    const pkgVersions = (0, tailwind_dependencies_1.getTailwindDependencies)();
    return (0, devkit_1.addDependenciesToPackageJson)(tree, {}, {
        autoprefixer: pkgVersions.autoprefixer,
        postcss: pkgVersions.postcss,
        tailwindcss: pkgVersions.tailwindcss,
    });
}
exports.addTailwindRequiredPackages = addTailwindRequiredPackages;
function updateApplicationStyles(tree, options, project) {
    let stylesEntryPoint = options.stylesEntryPoint;
    if (stylesEntryPoint && !tree.exists(stylesEntryPoint)) {
        throw new Error(`The provided styles entry point "${stylesEntryPoint}" could not be found.`);
    }
    if (!stylesEntryPoint) {
        stylesEntryPoint = findStylesEntryPoint(tree, options, project);
        if (!stylesEntryPoint) {
            throw new Error((0, devkit_1.stripIndents) `Could not find a styles entry point for project "${options.project}".
        Please specify a styles entry point using the "--stylesEntryPoint" option.`);
        }
    }
    const stylesEntryPointContent = tree.read(stylesEntryPoint, 'utf-8');
    tree.write(stylesEntryPoint, (0, devkit_1.stripIndents) `@tailwind base;
    @tailwind components;
    @tailwind utilities;

    ${stylesEntryPointContent}`);
}
exports.updateApplicationStyles = updateApplicationStyles;
function findStylesEntryPoint(tree, options, project) {
    // first check for common names
    const possibleStylesEntryPoints = [
        (0, devkit_1.joinPathFragments)(project.sourceRoot ?? project.root, 'styles.css'),
        (0, devkit_1.joinPathFragments)(project.sourceRoot ?? project.root, 'styles.scss'),
        (0, devkit_1.joinPathFragments)(project.sourceRoot ?? project.root, 'styles.sass'),
        (0, devkit_1.joinPathFragments)(project.sourceRoot ?? project.root, 'styles.less'),
    ];
    const stylesEntryPoint = possibleStylesEntryPoints.find((s) => tree.exists(s));
    if (stylesEntryPoint) {
        return stylesEntryPoint;
    }
    // then check for the specified styles in the build configuration if it exists
    const styles = project.targets?.[options.buildTarget].options?.styles;
    if (!styles) {
        return undefined;
    }
    // find the first style that belongs to the project source
    const style = styles.find((s) => typeof s === 'string'
        ? s.startsWith(project.root) && tree.exists(s)
        : s.input.startsWith(project.root) &&
            s.inject !== false &&
            tree.exists(s.input));
    if (!style) {
        return undefined;
    }
    return typeof style === 'string' ? style : style.input;
}
function addTailwindConfigPathToProject(tree, options, project) {
    const buildTarget = project.targets?.[options.buildTarget];
    if (!buildTarget) {
        throw new Error((0, devkit_1.stripIndents) `The target "${options.buildTarget}" was not found for project "${options.project}".
      If you are using a different build target, please provide it using the "--buildTarget" option.
      If the project is not a buildable or publishable library, you don't need to setup TailwindCSS for it.`);
    }
    if (buildTarget.options?.tailwindConfig &&
        tree.exists(buildTarget.options.tailwindConfig)) {
        throw new Error((0, devkit_1.stripIndents) `The "${buildTarget.options.tailwindConfig}" file is already configured for the project "${options.project}". Are you sure this is the right project to set up Tailwind?
      If you are sure, you can remove the configuration and re-run the generator.`);
    }
    buildTarget.options = {
        ...buildTarget.options,
        tailwindConfig: (0, devkit_1.joinPathFragments)(project.root, 'tailwind.config.js'),
    };
    (0, devkit_1.updateProjectConfiguration)(tree, options.project, project);
}
exports.addTailwindConfigPathToProject = addTailwindConfigPathToProject;
function addTailwindConfigFile(tree, options, project) {
    if (tree.exists((0, devkit_1.joinPathFragments)(project.root, 'tailwind.config.js'))) {
        throw new Error((0, devkit_1.stripIndents) `The "tailwind.config.js" file already exists in the project "${options.project}". Are you sure this is the right project to set up Tailwind?
      If you are sure, you can remove the existing file and re-run the generator.`);
    }
    (0, devkit_1.generateFiles)(tree, (0, devkit_1.joinPathFragments)(__dirname, '..', 'files', 'tailwind'), project.root, {
        relativeSourceRoot: (0, node_path_1.relative)(project.root, project.sourceRoot),
        template: '',
    });
}
exports.addTailwindConfigFile = addTailwindConfigFile;
//# sourceMappingURL=add-tailwind-helpers.js.map